/*
 * cc2511.h
 */

#ifndef INC_CC2511_H_
#define INC_CC2511_H_

/* Includes ------------------------------------------------------------------*/
#ifndef __STM32F4xx_HAL_H
#include "stm32f4xx_hal.h"
#endif
#include "main.h"

/*******************************************************************************
 * DEFINES
 */
/*
 * CLK @ 96Mhz, 30 cycles is roughly 3.5us
 * Adjust as needed.
 */
#define DELAY_CYCLES 30

#define DD_LOW  GPIOE->BSRR = GPIO_BSRR_BR4
#define DD_HIGH GPIOE->BSRR = GPIO_BSRR_BS4
#define DC_LOW  GPIOE->BSRR = GPIO_BSRR_BR5
#define DC_HIGH GPIOE->BSRR = GPIO_BSRR_BS5
#define RST_LOW  GPIOE->BSRR = GPIO_BSRR_BR6
#define RST_HIGH GPIOE->BSRR = GPIO_BSRR_BS6
#define IS_DD_HIGH ((GPIOE->IDR & (1 << 4)) != 0)
#define DELAY(x) for (uint32_t i = 0; i < DELAY_CYCLES * x; i++) __NOP()

/* Set programmer DD line as output */
#define SET_DD_OUTPUT do { \
    GPIOE->MODER &= ~(3UL << (4 * 2)); /* Clear mode bits for pin 4 */ \
    GPIOE->MODER |= (1UL << (4 * 2));  /* Set mode bits to output for pin 4 */ \
} while (0)
/* Set programmer DD line as input */
#define SET_DD_INPUT	(GPIOE->MODER &= ~(3UL << (4 * 2)));

// Start addresses on DUP (Increased buffer size improves performance)
#define ADDR_BUF0                   0x0000 // Buffer (512 bytes)
#define ADDR_DMA_DESC_0             0x0200 // DMA descriptors (8 bytes)
#define ADDR_DMA_DESC_1             (ADDR_DMA_DESC_0 + 8)

// Debug commands
#define CMD_CHIP_ERASE              0x10
#define CMD_WR_CONFIG               0x1D
#define CMD_RD_CONFIG               0x24
#define CMD_READ_STATUS             0x34
#define CMD_GET_PC					0x28
#define CMD_DEBUG_INSTR_1B          0x55
#define CMD_DEBUG_INSTR_2B          0x56
#define CMD_DEBUG_INSTR_3B          0x57
#define CMD_HALT					0x44
#define CMD_RESUME                  0x4C
#define CMD_BURST_WRITE             0x80
#define CMD_GET_CHIP_ID             0x68

// Debug status bit masks
#define STATUS_CHIP_ERASE_BUSY_BM   0x80 // New debug interface
#define STATUS_PCON_IDLE_BM         0x40
#define STATUS_CPU_HALTED_BM        0x20
#define STATUS_PM_ACTIVE_BM         0x10
#define STATUS_HALT_STATUS_BM       0x08
#define STATUS_DEBUG_LOCKED_BM      0x04
#define STATUS_OSC_STABLE_BM        0x02
#define STATUS_STACK_OVERFLOW_BM    0x01

// DUP registers (XDATA space address)
#define DUP_DBGDATA                 0x6260  // Debug interface data buffer
#define DUP_FCTL                    0x6270  // Flash controller
#define DUP_FADDRL                  0x6271  // Flash controller addr
#define DUP_FADDRH                  0x6272  // Flash controller addr
#define DUP_FWDATA                  0x6273  // Clash controller data buffer
#define DUP_CLKCONSTA               0x759E  // Sys clock status
#define DUP_CLKCONCMD               0x75C6  // Sys clock configuration
#define DUP_MEMCTR                  0x70C7  // Flash bank xdata mapping
#define DUP_DMA1CFGL                0x70D2  // Low byte, DMA config ch. 1
#define DUP_DMA1CFGH                0x70D3  // Hi byte , DMA config ch. 1
#define DUP_DMA0CFGL                0x70D4  // Low byte, DMA config ch. 0
#define DUP_DMA0CFGH                0x70D5  // Low byte, DMA config ch. 0
#define DUP_DMAARM                  0x70D6  // DMA arming register

// DMA channels used on DUP
#define CH_DBG_TO_BUF0              0x01   // Channel 0
#define CH_BUF0_TO_FLASH            0x02   // Channel 1

#define CMD_BURST_WRITE             0x80
#define CMD_GET_CHIP_ID             0x68

#define FCTL						0xDFAE	// Flash Control

/*/******************************************************************************
 * TYPEDEFS
 */
/* Set programmer DD line as output */
#define SET_DD_OUTPUT do { \
    GPIOE->MODER &= ~(3UL << (4 * 2)); /* Clear mode bits for pin 4 */ \
    GPIOE->MODER |= (1UL << (4 * 2));  /* Set mode bits to output for pin 4 */ \
} while (0)
/* Set programmer DD line as input */
#define SET_DD_INPUT	(GPIOE->MODER &= ~(3UL << (4 * 2)));
//! Low nibble of 16bit variable
#define LOBYTE(w)           ((unsigned char)(w))
//! High nibble of 16bit variable
#define HIBYTE(w)           ((unsigned char)(((unsigned short)(w) >> 8) & 0xFF))
//! Convert XREG register declaration to an XDATA integer address
#define XREG_TO_INT(a) ((uint8_t)((a) & 0xFF))

/******************************************************************************
 * FUNCTIONS
 */
void cc2511_debug_init(void);
void cc2511_clock_init(void);
uint16_t cc2511_read_chip_id(void);
uint8_t cc2511_read_status(void);
void cc2511_chip_erase(void);
void cc2511_halt_cpu(void);
void cc2511_resume_cpu(void);
void cc2511_burst_write_block(uint8_t *src, uint8_t num_bytes);
void cc2511_write_debug_byte(uint8_t data);
uint8_t cc2511_read_debug_byte(void);
uint8_t cc2511_read_xdata_memory(unsigned short  address);
uint8_t cc2511_wait_dup_ready(void);
uint8_t cc2511_debug_command(uint8_t cmd, uint8_t *cmd_bytes, uint8_t num_cmd_bytes);
void cc2511_write_xdata_memory(unsigned short address, unsigned char value);
void cc2511_write_flash_memory_block(uint8_t *src, uint32_t start_addr, uint8_t num_bytes);
void cc2511_write_xdata_memory_block(int address, uint8_t *values, uint8_t num_bytes);
void cc2511_read_flash_memory_block(unsigned char bank,unsigned short flash_addr, unsigned short num_bytes, unsigned char *values);
uint8_t cc2511_write_code_memory(uint16_t address, uint8_t buffer[], int len);
uint8_t cc2511_verify_code_memory(uint16_t address, uint8_t buffer[], int len);

#endif /* INC_CC2511_H_ */
